;******************************************************************************
;  File Name    : pwm_long_pulse_12f683.asm
;  Version      : 1.2
;
;  Description  : PWM on Pin 5 (CCP1)  
;  Target       : Microchip PIC 12F683 Microcontroller
;  Clock        : Internal RC 4MHz (instruction execution time: 1usec)
;  Compiler     : Microchip Assembler (MPASM)
;  IDE          : Microchip MPLAB IDE v8.60
;  Programmer   : PICKit3
;  Last Updated : 15. November 2013
;******************************************************************************

      list     P=12F683, ST=OFF   ; Turnoff Symbol Table in List file.

      errorlevel  -302        ; Ignore error message when storing to Bank 1.

;==============================================================================

       #include   <p12f683.inc>

	__CONFIG  _INTRC_OSC_NOCLKOUT & _WDT_OFF & _PWRTE_ON & _MCLRE_ON & _CP_OFF & _CPD_OFF & _BOD_OFF & _IESO_OFF & _FCMEN_OFF 

    ; ------------------------------------
    ; Configuration Bit Settings Explained
    ; ------------------------------------
    ; Internal RC Oscillator, No Clock Out
    ; Watch Dog Timer = OFF
    ; Power Up Timer = ON
    ; MCLR = ON
    ; Code Protect = OFF
    ; Data EE Read Protect = OFF
    ; Brown Out Detect = OFF
    ; Internal External Switch Over Mode (2-speed Startup) = OFF
    ; Monitor Clock Fail-safe = OFF

    ;==========================================================================

;*****[ Data Storage Reg's ]*****
		UDATA
;
dc1		res 1
dc2		res 1
dc3		res 1

;==================================================================================

         ORG     0           ; Program origin at mem location 0.
;----------------------------------------------------------------------------------
Initialize
            BANKSEL OSCCON          ; Switch to Bank 1.
            MOVLW   b'01100001'     
;                      110          4MHz Clk, 
;                         0         IntOsc, 
;                            1      SysClk via IntOsc
            MOVWF   OSCCON

            BANKSEL CMCON0          ; Switch to Bank 0.
            MOVLW   b'00000111'     ; Turn off Comparator.
            MOVWF   CMCON0   
            BANKSEL ANSEL           ; Switch to Bank 1.
            CLRF    ANSEL           ; Set I/O pins to Digital.

            MOVLW   b'00001000'     ; Define inputs & outputs: all output expect gp3 (gp3=input allways).
            MOVWF   TRISIO
            MOVLW   b'11000011'     
;                     1             Disable pull-ups, 
;                         0         Prescaler -> TMR0,
;                          011      TMR0 prescaler = 16.
            MOVWF   OPTION_REG      
    
            BANKSEL GPIO            ; Switch to Bank 0.
         	movlw   b'00000000'     ; Set GPIO initial states.
            movwf   GPIO            ; 
    
 
            MOVLW   B'00001101'		
;                       00          Set bits 4-5 (2 LSB's of Duty Cycle) to 00.
;                         11        Set CCP1 as PWM
;                           0x      PWM: 0=active high  1=active low  
            MOVWF   CCP1CON         
    
    ; Set highest PWM value (resolution), 10-bit but only 256 discrete Duty Cycles.
            BANKSEL PR2             ; Bank 1
            MOVLW   d'255'          ; 10-bit res. but 256 Duty Cycles (Duty Cycle LSB = 00)
            MOVWF   PR2             ; Establishes Timer2 Period.

   
    ; Set prescaler to 16 for a PWM freq. of 244Hz (based on 4MHz clock).
    ; Since the human eye cannot perceive flicker above 120Hz, 244Hz is OK.
            BANKSEL T2CON           ; Bank 0
            MOVLW   B'00000010'
;                          0        Timer2 is now OFF,
;                           1x      Prescaler is 16.
            MOVWF   T2CON           
    
            CLRF    CCPR1L          ; Clear PWM Reg.1 LO Byte (sets PWM Duty Cycle to zero)
                                    ; (CCPR1L contains the 8 MSB's of the Duty Cycle.)
            BSF     T2CON, TMR2ON   ; Turn on Timer2.

			goto	main_loop

;==================================================================================




;****** MAIN PROGRAM ****************************************************
MAIN	CODE

;****** Main loop
main_loop

		BANKSEL	CCPR1L			; CCPR1L = MSB (most signifcant bits) of duty cycle
		movlw	b'00000000'	    ; set to 0
		movwf	CCPR1L



Begin
		movlw    .1
		call     delay10

		banksel CCPR1L
        INCF    CCPR1L, F       ; Increase Duty Cycle (make LED brighter).
        MOVLW   b'11111111'
		banksel CCPR1L
        XORWF   CCPR1L, W   
        SKPZ                    ; Have we incremented up to 255 yet?   
        GOTO    Begin           ; No.  Keep incrementing (increasing brightness).

    							; Yes...
		movlw    .200			; Leave lights on for a while, with full brightness 
		call     delay10

		movlw    .200
		call     delay10


Fadeout
		movlw    .1
		call     delay10

		banksel CCPR1L
        DECFSZ  CCPR1L, F       ; Decrease Duty Cycle (make LED dimmer).
        GOTO    Fadeout         ; Keep decrementing until 0 (LED OFF).

		movlw    .10
		call     delay10

		goto	main_loop
	






;****** SUBROUTINES ****************************************************
SUBS	CODE
;****** Variabel delay: 10ms ... 2550ms
;
;  Delay = W x 10 ms
;
delay10_R
delay10
		banksel	dc3
		movwf	dc3			; delay = 1+Wx(3-10009+3)-1+4 = W x 10.015ms
dly2	movlw	.13			; repeat inner loop 13 times
		movwf	dc2			; --> 13x(767+3)-1 = 10009 cycles
		clrf	dc1
dly1	decfsz	dc1,f
		goto 	dly1
		decfsz	dc2,f		; end middle loop
		goto	dly1
		decfsz	dc3,f		; end outer loop
		goto	dly2
		
		retlw	0


  	

		END                       ; directive 'end of program'



